function [sys,x0,str,ts] = kite(t,x,u,flag)
%   S-function describing a kite model as given in Chapter 8 of
%
%     Moritz Diehl:
%     "Real-Time Optimization for  Large Scale Nonlinear Processes"
%     (PhD-thesis, University of Heidelberg). 
%     Fortschr.-Ber. VDI Reihe 8, Mess , Steuerungs-  und
%     Regelungstechnik, Vol. 920, VDI Verlag, Duesseldorf (2002).
%
%     http://www.ub.uni-heidelberg.de/archiv/1659/
%
%   INPUT:
%     u     = psi (deg)
%
%   STATES:
%     x(1)  = theta
%     x(2)  = phi
%     x(3)  = theta dot
%     x(4)  = phi dot
%
%   OUTPUTS:
%     y(1)  = theta
%     y(2)  = phi
%     y(3)  = theta dot
%     y(4)  = phi dot
%
%
%   Copyright 2003 Moritz Diehl
%
%   For a description of the model see also:
%
%     Diehl, Magni, Scattolini:
%     "Online NMPC of a Looping Kite using Approximate Infinite
%     Horizon Closed Loop Costing", 
%     IFAC CSD 2003, Sep 7-11, Bratislava (2003).
%
%


switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes;

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  case 1,
    sys=mdlDerivatives(t,x,u);

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys=mdlOutputs(t,x,u);

  %%%%%%%%%%%%%%%%%%%
  % Unhandled flags %
  %%%%%%%%%%%%%%%%%%%
  case { 2, 4, 9 },
    sys = [];

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end
% end kite

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes()

sizes = simsizes;
sizes.NumContStates  = 4;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 4;
sizes.NumInputs      = 1;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
x0  = [1.00666773429988   0.02690485058294   0.36755458810190   0.46896608940152];
str = [];
ts  = [0 0];

% end mdlInitializeSizes
%
%=============================================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%=============================================================================
%
function sys=mdlDerivatives(t,x,u)
radius=50;
mass=1;
wo=[6;0;0];
rho=1.2;
area=0.5;
g=9.81;
CL = 1.5;
CD = 0.2907;


psi=u/180*3.1415927;

theta    = x(1);
phi      = x(2);
thetadot = x(3);
phidot   = x(4);

if (theta>3.14159/2)
   error(['***** KITE CRASHED !!! *****']);
end

F_aer  = Force_aer(radius,theta,phi,thetadot,phidot,psi, rho,area, CL, CD, wo);
F_grav = [g*mass*sin(theta); 0;-g*mass*cos(theta)];
F      = F_aer+F_grav; % Force in local coordinates 

thetadotdot = F(1)/(mass*radius)+sin(theta)*cos(theta)*phidot*phidot;
phidotdot   = F(2)/(mass*radius*sin(theta)) - 2.0*phidot*thetadot/tan(theta); 

sys(1)=thetadot;
sys(2)=phidot;
sys(3)=thetadotdot;
sys(4)=phidotdot;


% end mdlDerivatives
%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

sys(1:4) = x;

% end mdlOutputs
function [F_aer, p, el, et]=Force_aer(radius, theta ,phi, thetadot, phidot, psi, ...
				   rho, area, CL, CD, wo);

% local system relative to earth system
 
ep=[...
    cos(theta)*cos(phi)  -sin(phi)    sin(theta)*cos(phi)
    cos(theta)*sin(phi)  cos(phi)     sin(theta)*sin(phi)   
    -sin(theta)          0            cos(theta)
   ];

% in the sequel all vectors are in local system!

wp=ep'*wo;

% calculate wind due to movement (in local coordinates)
vp=[...
    radius*thetadot
    radius*sin(theta)*phidot
    0
  ];
vp=-vp;

%calculate effective wind we (in local coordinates)
wep=vp+wp;

epp=zeros(3,3);
% epp(:,1) pointing in projected direction of effective wind wep 
epp(:,1)=wep;
epp(3,1)=0;
wbar=norm(epp(:,1));
if (wbar<1e-8) 
   error(['Kite orientation not defined (no effective wind tangential to sphere)']);
end
epp(:,1)=epp(:,1)/wbar;

% epp(:,3) perpendicular to tangent plane
epp(:,3)=[0;0;1];

% epp(:,2) completing right-hand sided system: epp(:,2) =  epp(:,3)  x  epp(:,1) 
epp(:,2)=outprod(epp(:,3), epp(:,1));

% translate wep in new system
wepp=epp'*wep;
% (effective wind in new local coordinates: wepp=[wbar;0;wep(3)] )

% right wingtip wt in new local coordinates epp 
%     ATTENTION: only possible if 
%     wepp(3)*tan(psi)<wepp(1)
ratio=wepp(3)*tan(psi)/wepp(1);
if (abs(ratio)>1)
   error(['Kite orientation not defined (control angle psi too big or wbar' ...
	  ' too small)']);
end   
eta = asin(ratio);

etpp=[cos(psi)*(-sin(eta));cos(psi)*(cos(eta));sin(psi)];
% Remark:
%    "etpp" is the vector "et" seen in the new local coordinates epp
%
% Derivation: etpp perpendicular to effective wind, ie.
%     dot(wepp,etpp)=0   <==> 
%     wepp(1)*cos(psi)*(-sin(eta)) + wepp(3)*sin(psi) = 0   <==> 
%     sin(eta) = wepp(3)*sin(psi)/wepp(1)/cos(psi)

% lift direction ldpp = wepp/|wepp| x etpp
ldpp=outprod(wepp/norm(wepp), etpp);

% Turn etpp into local system
etp=epp*etpp;

% Turn ldpp into local system
ldp=epp*ldpp;

% vectors el and et in local coordinates:
elp=wep/norm(wep);

% calculate aerodynamic force Fa
weabs=norm(wep);
Pdyn=weabs*weabs*1/2*rho;

Lift=Pdyn*area*CL;
Drag=Pdyn*area*CD;

F_aer=Lift * ldp + Drag * elp;

%
% Remainder only for possible use in 3-D graphics:
%

% position of kite
p=radius*ep(:,3);
% vectors el and et in earth coordinates:
el=ep*elp;
et=ep*etp;

% end of Force_aer




function c=outprod(a, b)
  c(1)=  a(2)*b(3) - a(3)*b(2);
  c(2)=  a(3)*b(1) - a(1)*b(3);
  c(3)=  a(1)*b(2) - a(2)*b(1);
  c=c(:);
