function [sys,x0,str,ts] = uref(t,x,u,flag)
%   S-function without inputs, whose output is an open loop control 
%   of a reference trajectory for 
%   the looping kite model as given in Chapter 8 of
%
%     Moritz Diehl:
%     "Real-Time Optimization for  Large Scale Nonlinear Processes"
%     (PhD-thesis, University of Heidelberg). 
%     Fortschr.-Ber. VDI Reihe 8, Mess , Steuerungs-  und
%     Regelungstechnik, Vol. 920, VDI Verlag, Duesseldorf (2002).
%
%     http://www.ub.uni-heidelberg.de/archiv/1659/
%
%
%     y = psi (deg)
%
%
%   Copyright 2003 Moritz Diehl
%
%

switch flag,
  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  % Initialize the states, sample times, and state ordering strings.
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes;

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  % Return the outputs of the S-function block.
  case 3
    sys=mdlOutputs(t,x,u);

  %%%%%%%%%%%%%%%%%%%
  % Unhandled flags %
  %%%%%%%%%%%%%%%%%%%
  % There are no termination tasks (flag=9) to be handled.
  % Also, there are no continuous or discrete states,
  % so flags 1,2, and 4 are not used, so return an emptyu
  % matrix 
  case { 1, 2, 4, 9 }
    sys=[];

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Unexpected flags (error handling)%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Return an error message for unhandled flag values.
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end uref

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes()

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 1;
sizes.NumInputs      = 0;
sizes.DirFeedthrough = 0; 
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
str = [];
x0  = [];
ts  = [0 0];   % inherited sample time

% end mdlInitializeSizes

%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u)
u_r_coeff=[...
-7.309120017868209e-04  8.822696214582962e+00  2.150034179595898e+00  -5.985955751149583e-04  -4.456369915389231e-04  2.870523161552530e-01  -9.366678480535263e-02  -4.182307830411928e-04  -3.928250141850727e-05  2.514908326966904e-02  1.166346492697405e-02  -2.485270475186893e-04  3.368195746191426e-05  5.908299552792435e-03  -1.390898045086762e-03  
];

sinvec=calc_sinvec(t);
sys=0;
for j=1:15
  sys = sys + u_r_coeff(1,j)*sinvec(j);
end
% end mdlOutputs



